/****************************************************************
 *								*
 * Line overrating and maitenance fitness function              *
 *								*
 * File: fitness.c	$Revision: 1.54 $      		       	*
 *								*
 * Author: W. Langdon 
 *								*
 * Started: 26 Oct 1994 					*
 *								*
 ****************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <float.h>
#include <assert.h>
#include "loadflow.h"
#include "netchk3.c"

/*Modifications (in reverse order)

WBL 13 Jun 96 Revert to original (ie no contingencies) SWales cf r1.7
              But with modern interfaces, so can use ltcoef and netchk3

WBL  4 May 96 KRevert to four node problem

WBL 21 Apr 96 Roll own hash, nb assume network is always the same (NNLK,NNLJ)

WBL 20 Apr 96 Convert to C++ and use LEDA hash

WBL 26 Feb 96 For GPgrid dont do loadflow cals on contingencies

WBL 13 Feb 96 Restore powerflow display changes in r1.38

WBL 13 Feb 96 Restore changes in r1.36 and combine with changes in
	       r1.40 Ie C version of ltcoef

	       Prevent FitnessWeek calculating lineflows if a
	       contigency causes a loss of load or generation. This
	       allows it to avoid using loadflow and instead use
	       ltcoef, for contingencies.

	       Fitness calculate contingency costs as either as actual
	       line costs or, if FitnessWeek has not calculated them,
	       as line costs of base case plus load shed.

	       Use simpler version of newnet which avoids renumbering
	       lines by marking all isolated ones as being in maintenance.

WBL 30 Jan 96  Replace netchk and nodchk by netchk3

WBL 23 Jan 96  Add shed_costweek

WBL 22 Jan 96  Add calling ltcoef

WBL 24 Nov 95  Use globals linename and nodename

WBL 24 Nov 95  Make S_factor and Costing parameters of FitnessWeek

WBL 22 Nov 95  Make network splits less dramatic

WBL 26 Oct 95  Make intermediate results produced by FitnessWeek available

WBL 22 Oct 95  Include display code from fitness1.c
Include line loadings on reminder of network should nodes be isolated

WBL 15 Oct 95  Add support for contigencies

WBL  1 Mar 95  In FitnessWeek allow for more than 4 nodes should netchk fail
               BUGFIX Load_shed to use ABS rather than abs. NB probably changes
	       lots of earlier fitness calculation on fournode problem.

WBL 27 Nov 94  Split Fitness into Fitness and FitnessWeek

WBL 13 Nov 94  Add using netchk and nodchk. Base Load_sheding on them rather
               than flow imbalance
*/


/*#define rating   0.85*/
#define K_factor 4000

/* ABS gives the modulus of the given argument */
#define ABS(x)  ((x)>=0? (x) : -(x))

extern int loadflow_call1;
extern int loadflow_call2;
extern int ltcoef_call1;
extern int ltcoef_call2;

void Display (int nn, int nl, int w, int* NNLK, int* NNLJ,
	      float* FLOW, float* LINERATING, BOOL* MAINT,
	      float cost, float shed, float fitness )
{
	int i, j, k;
	
	printf ("\tDC Loadflow Results Week %2d\n",w);
	printf ("\t---------------------------\n\n\n");
	printf ("    Line       Node  To   Node\t\t   Flows\tLoading\n\n");
	
	for (i=1; i<=nl; i++) {
	   float LOAD;
	   k = NNLK[i];
	   j = NNLJ[i];
	   LOAD = ABS(FLOW[i]/LINERATING[i]) * 100;
	   printf("%3d %6s %3d %6s %3d %6s\t%12.6f\t%4.1f%%",
		  i,linename[i],k,nodename[k],j,nodename[j],FLOW[i],LOAD);
/*	   printf("%d\t%d\t%d\t%f\t%3.1f%%",i,k,j,FLOW[i],LOAD);*/
	   if(MAINT[i]) printf(" M");
	   printf("\n");
	}

/*
        printf ("\n\tBus No.        Imbalance\t\t\tLine costs\n\n");
        for (i=1; i<=nn; i++) {
                printf ("\t%d\t\t%4.1fMW\n",i,IMBALANCE[i]);
        }
*/
	printf("shed %4.1fMW\tcost %f\tfitness %f\n\n", shed, cost, fitness);
	return;
}


float Costing(int nl, float* LOAD, float* LINERATING)
{
int i;
float cost = 0.0;

for ( i = 1; i<= nl; i++){
	float over;
	over = ABS(LOAD[i]) - (LINERATING [i] * rating);
        if ( over > 0.0 ) {
		if(FITNESS_DISPLAY) printf("\t%s%6.0f",linename[i],over);
		cost += over;
	}
     }

if(FITNESS_DISPLAY) printf(" =%6.0f\n",cost);

return cost;
}

float Load_sheding(const int nn, const float P[NN], const BOOL splnod[NN] )
{
int i;
float shed = 0.0;

for ( i = 1; i<= nn; i++){
	if ( splnod[i] ) shed += ABS(P[i]) * 100.0; /* P is in 100MW */
     }
return shed;
}


void newnet(const int nl,         
            const int  NNLK[NL],   const  int NNLJ[NL],   
            const BOOL MAINT[NL],  const BOOL splnod[NN],
/* Outputs */
	    BOOL  maint_[NL] )
{ 
int n; 
  
for(n=1;n<=nl;n++) { 
	maint_[n] = ((splnod[NNLK[n]] || splnod[NNLJ[n]]))? TRUE: MAINT[n];
	}
}/*end newnet*/

void  newnetflow(int nn, int nl, float P[NN], int NNLK[NL], int NNLJ[NL], 
		 float XL[NL], float LINERATING[NL], BOOL MAINT[NL], 
		 BOOL splnod[NN],
		 float Costing(int nl, float LOAD[NN], float LINERATING[NL]),
		 float S_factor,
/* outputs */
		 float* cost, float FLOW[NL])
{
	BOOL maint_[NL];
	float dummy[NL];
	BOOL save_display = FITNESS_DISPLAY;

/* Will only be called if the old network is split and has shed
   load. Under these circumstances we cant use ltcoef, instead must
   use Loadflow */

	newnet(nl, NNLK, NNLJ, MAINT, splnod,
	       maint_);

/* Check nolonger required as going back to Loadflow.c WBL 13-Feb-1996 */
/*	if(nn_<nn/2) { /* SDRS2 fails if nn_ <= 1 
		int i;
		for(i=1;i<=nn;i++) splnod[i]=TRUE;
		shed = Load_sheding(nn, P, splnod);
		if(FITNESS_DISPLAY)
			printf(" =%6.0f (Too much shed)\n",(S_factor*shed));
	}
	else { */
	        loadflow_call1++;
		Loadflow (nn, nl, P, NNLK, NNLJ, XL, maint_,
			  FLOW, dummy); 
		FITNESS_DISPLAY=FALSE;
		*cost = Costing (nl, FLOW, LINERATING); 
		FITNESS_DISPLAY = save_display;
		if(FITNESS_DISPLAY){
		int n;
		float shed;
		shed = Load_sheding(nn, P, splnod);		
		printf(" =%6.0f (Isolated nodes ",(S_factor*shed)+(*cost));
		for(n=1;n<=nn;n++)if(splnod[n])
			printf(" %s%6.0f",nodename[n],P[n]*100.0);
		if(*cost>0) {
			printf(", Overloaded lines ");
		        Costing (nl, FLOW, LINERATING); /*just too display*/
		}
 		printf(")\n");
 		for(n=1;n<=nl;n++) {
 			printf(" %s%6.0f/%6.0f",
 			       linename[n], FLOW[n], LINERATING[n]*rating);
 			if(MAINT[n]) printf("M "); else printf("  ");
 			if(n%3==0) printf("\n");
		}
		printf(")\n");
	}
};/* end newnetflow */

float load_shed(int nn, int nl, float P[NN],
                int NNLK[NL], int NNLJ[NL], BOOL MAINT[NL] )
{
	BOOL splnod[NN];
	if (netchk3(nl, nn, NNLK, NNLJ, MAINT, splnod)) {
		return 0.0;
		/* niether split network nor isolated nodes */
	}
	return Load_sheding(nn, P, splnod);

}/*end load_shed*/

float FitnessWeek(int nn, int nl, float P[NN],
	      int* NNLK, int* NNLJ,
	      float* XL, float* LINERATING, BOOL MAINT[NL],
	      float Costing(int nl, float LOAD[NN], float LINERATING[NL]),
	      float S_factor, BOOL contingency,
	      /*base data*/
	      double xx[NN*2], double flow[NL], 
	      int ntovar[NN], double Q[NN*NN/2],
	      /* Display outputs */
	      float* line_costs, float* load_shed, float FLOW[NL],
	      BOOL* netsplit, BOOL splnod[NN] ) {

	float IMBALANCE[NL];
	float c, shed;

	*netsplit = FALSE;
	c = 0;
	shed = 0;
	if (!netchk3(nl, nn, NNLK, NNLJ, MAINT, splnod)) {
		*netsplit = TRUE;
	if (!netchk(nl, NNLK, NNLJ, MAINT, TRUE)) {
		int i;
		BOOL all[NN];
		for(i=1;i<=nn;i++) all[i]=TRUE;
		shed = Load_sheding(nn, P, all);
	}
	else
	if (!nodchk(nn, nl, NNLK, NNLJ, MAINT, splnod))
		shed = Load_sheding(nn, P, splnod );
        }
	else if(!contingency) {
		double susc[NL];
		int i, numout, kout[NL];
                double flow_out[NL];

		numout = 0;
		memset(kout,0,sizeof(kout));
		for(i=1;i<=nl;i++){
			susc[i] = -1.0/XL[i];
			if(MAINT[i]) {
				numout++;
				kout[numout]=i;
			}
		}

                if(numout==0) {
                        for(i=1;i<=nl;i++) FLOW[i] = flow[i-1];
                }
                else if(numout<=MXCLIN) { /*call FORTRAN*/
			ltcoef_call2++;
			ltcoef(nl, nn, susc, NNLK, NNLJ,
			       numout, kout, MAINT,
			       &xx[-1], &flow[-1], &ntovar[-1], &Q[-1],
			       flow_out);
			for(i=1;i<=nl;i++) FLOW[i] = flow_out[i];
                } 
		else { /* do complete calculation */
		loadflow_call2++;
		Loadflow (nn, nl, P, NNLK, NNLJ, XL, MAINT, FLOW,
			  IMBALANCE); 
		}
		c = Costing (nl, FLOW, LINERATING); 
	} else if(FITNESS_DISPLAY) printf("\n");

	*load_shed = shed;
	*line_costs = c;
	return (S_factor*shed) + c;

}/*end FitnessWeek*/


float Fitness(int nn, int nl, int nw, int ncontingencies, float P[NW][NN],
	      int* NNLK, int* NNLJ,
	      float* XL, float* LINERATING, BOOL MAINT[NW][NL],
	      BOOL contingency[NCONT][NL], float scale[NCONT],
	      /* Base data */
	      double xx[NW][NN*2], double flow[NW][NL], 
	      int ntovar[NN], double Q[NN*NN/2],
	      /* display outputs */
	      int* lines_maint, float* line_costs, float* load_shed,
	      float contingency_cost[NCONT] ) {
int w, i, k;
float fit, c, shed;
float total_fit = 0.0;
float total_c = 0.0;
float total_shed = 0.0;
int benefit = 0;
BOOL line_maintained [NL];
float fault_rating[nl];
const float S_factor = 5.0;
float base_line_costs[NW];

memset(line_maintained,0,sizeof(line_maintained));

for (w=1; w<=nw; w++) {	
	float FLOW[NL];
	BOOL dummy1; BOOL dummy2[NN];

	fit = FitnessWeek(nn, nl, P[w], NNLK, NNLJ, XL, LINERATING, MAINT[w],
	      Costing, S_factor, FALSE,
	      /*base data*/
	      xx[w], flow[w], ntovar, Q,
	      /* Display outputs */
	      &c, &shed, FLOW, &dummy1, dummy2 );
	total_fit += fit;
	base_line_costs[w] = c;
	total_c += c;
	total_shed += shed;
	for(i=1;i<=nl;i++)
		if(MAINT[w][i]) line_maintained[i] = TRUE;
/*	Display (nn, nl, w, 
		 NNLK, NNLJ, FLOW, LINERATING, MAINT[w], c, shed, fit); 
*/
}

for(i=1;i<=nl;i++)
	if(line_maintained[i]) benefit++;

*lines_maint = benefit;
*line_costs  = total_c;
*load_shed   = total_shed;

/*great scope for CPU and memorty savings possible here WBL 16/10/95 */
for(i=1; i<=nl; i++) fault_rating[i] = LINERATING[i]/rating;
for(k=1; k<=ncontingencies; k++) {
contingency_cost[k] = 0.0;
/*printf("Now considering contigency %d\n", k);*/
for (w=1; w<=nw; w++) {	
	float fit, c, shed;
	BOOL maint[nl];
	float FLOW[NL];
	BOOL dummy1; BOOL dummy2[NN];

	for(i=1; i<=nl; i++) maint[i] = MAINT[w][i] || contingency[k][i];

	fit = FitnessWeek(nn, nl, P[w], NNLK, NNLJ, XL, fault_rating, maint,
	      Costing, S_factor, TRUE,
	      /*base data*/
	      xx[w], flow[w], ntovar, Q,
	      /* Display outputs */
	      &c, &shed, FLOW, &dummy1, dummy2 );
	if(shed != 0) {/* fit and c NOT calculated if load shed*/
		fit = base_line_costs[w] + shed*S_factor;
	}
	contingency_cost[k] += fit;
	total_fit += fit*scale[k];
/*****	total_c += c;
 *****	total_shed += shed;
 *****	for(i=1;i<=nl;i++)
 *****		if(MAINT[w][i]) line_maintained[i] = TRUE; */
/*	Display (nn, nl, w, 
		 NNLK, NNLJ, FLOW, fault_rating, maint, c, shed, fit); 
*/
}}/*end contingencies*/

/*printf("Total fitness %f\n",K_factor * (nl - benefit) + total_fit);*/

return (K_factor * (nl - benefit) + total_fit);

}/*end Fitness*/
