/****************************************************************
 *								*
 * DC Loadflow program 	$Revision: 1.32 $                        *
 *								*
 * File: loadflow.h	include file   				*
 *								*
 * Author: W. Langdon                                           *
 *								*
 * Started: 22 Oct 1994 					*
 *								*
 ****************************************************************/

/* Defines maximum number of network nodes and maximum number
   of network interconnections
        NB: in C arrays start at zero so these must be atleast one
   more than required size 
*/

#define NN	31
#define NL 	43
/* nb also update DEF_CHROMOSOME_LENGTH in qgatypes.h  and
   MAXN and MAX in fortran include file PARAM  */

#define NW      54

#define NCONT   55

/* must tally with value in rankone.jan/PARAM */
#define MXCLIN   9

#define nm_weeks 4

/* Ratio between quoted post fault continious rating (used in fn.cost
   and sw.cost, except for factor of 2) and in service use */
#define rating  0.85

#define BOOL int

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

/* ABS gives the modulus of the given argument */
#define ABS(x)  ((x)>=0? (x) : -(x))

void Loadflow(int nn, int nl, float* P,
	      int* NNLK, int* NNLJ, float* XL, BOOL* MAINT,
	      /*output*/
	      float* FLOW, float* IMBALANCE );

BOOL netchk   (int nl, int NNLK[NL], int NNLJ[NL], BOOL MAINT[NL], BOOL print);

BOOL netchk3  (int nl, int nn,  int NNLK[NL], int NNLJ[NL], BOOL MAINT[NL], 
	       /*output*/
	      BOOL splnod[NN] );

BOOL nodchk   (int nn, int nl, int NNLK[NL], int NNLJ[NL], BOOL MAINT[NL], 
	      /*output*/
	      BOOL splnod[NN] );

float Load_sheding(const int nn, const float P[NN], const BOOL splnod[NN] );

float load_shed(int nn, int nl, float P[NN],
		int NNLK[NL], int NNLJ[NL], BOOL MAINT[NL] );

extern BOOL FITNESS_DISPLAY;
extern char* linename[NL];
extern char* nodename[NN];
 
float FitnessWeek(int nn, int nl, float P[NN],
	      int* NNLK, int* NNLJ,
	      float* XL, float* LINERATING, BOOL MAINT[NL],
	      float (*Costing)(int nl, float LOAD[NN], float LINERATING[NL]),
	      float S_factor, BOOL contingency,
	      /*base data*/
	      double xx[NN*2], double flow[NL], 
	      int ntovar[NN], double Q[NN*NN/2],
	      /* Display outputs */
	      float* line_costs, float* load_shed, float FLOW[NL],
	      BOOL* netsplit, BOOL splnod[NN] );

float Fitness(int nn, int nl, int nw, int ncontingencies, float P[NW][NN],
	      int* NNLK, int* NNLJ,
	      float* XL, float* LINERATING, BOOL MAINT[NW][NL],
	      BOOL contingency[NCONT][NL], float scale[NCONT],
	      /* Base data */
	      double xx[NW][NN*2], double flow[NW][NL], 
	      int ntovar[NN], double Q[NN*NN/2],
	      /*output*/
	      int* lines_maint, float* line_costs, float* load_shed,
	      float contingency_cost[NCONT] );

float Costing(int nl, float LOAD[NN], float LINERATING[NL]);

void Schedule (int nn, int nl, int nw, int ncontingencies, float P[NW][NN],
	       int NNLK[NL], int NNLJ[NL], float XL[NL], float LINERATING[NL],
	       BOOL contingency[NCONT][NL], float scale[NCONT],
	       /* Readonly Base line data */
	       double xx[NW][NN*2], double flow[NW][NL], 
	       int ntovar[NN], double Q[NN*NN/2],
	       /* GA Chrome */
	       int order[NL],
	       /* outputs */
	       BOOL MAINT[NW][NL] );

char* Schedule_version();

void Display6 (int nl, int w, int line, BOOL MAINT[NL], 
	      float FLOW[NL], float cost);

void ReadParameters (char filename[32], 
		     int* nn, int* nl, int*nw, int*ncontingencies,
		     float P[NW][NN], int* NNLK, int* NNLJ, float* XL,
		     float* LINERATING,
		     BOOL contingency[NCONT][NL], float scale[NCONT],
		     double xx[NW][NN*2], double flow[NW][NL], 
		     int ntovar[NN], double Q[NN*NN/2]
);

void display_maint(int nl, int nw, BOOL MAINT[NW][NL]);

BOOL readnet(char filename[32], 
             int nn, int nl, int NNLK[NL], int NNLJ[NL],
             /* outputs */
             char* linename[NL], char* nodename[NN]);

void ltcoef(int nl, int nn, 
	    double susc[NL], int NNLK[NL], int NNLJ[NL], 
	    int numout, int kout[NL], int MAINT[NL], 
	    /* Readonly Base line data */
	    double xx[NN*2], double flow[NL], 
	    int ntovar[NN], double Q[NN*NN/2],
	    /* outputs */
	    double flownw[NL] );
