/****************************************************************
 *								*
 * DC Loadflow program 						*
 *								*
 * File: loadflow.c	main code      				*
 *								*
 * Author: W. Langdon 
 *         (from J.E. Hodgson (Fortran original by A. Ekwue))  	*
 *								*
 * Started: 22 Oct 1994 					*
 *								*
 ****************************************************************/

#define version "23 Oct 1994 $Revision: 1.4 $"

/*Includes main function


Modifications (in reverse order)

WBL 24 Oct 94  Display IMBALANCE

WBL 23 Oct 94  Adapt to allow lines to ne taken out for maintenance
*/

#include <stdio.h>
#include "loadflow.h"

/* ABS gives the modulus of the given argument */
#define ABS(x)  ((x)>=0? (x) : -(x))

/* This function calculates the net power, P, flowing to or from
 * each busbar (or node) */
void CalcP (int nn, float* PGEN, float* PLOAD,
	    float* P)
{
	int i;

	for (i=1; i<=nn; i++) {
		P[i] = PGEN[i] - PLOAD[i];
	}
}

void Display (int nn, int nl, int* NNLK, int* NNLJ,
	      float* FLOW, float* LINERATING, BOOL* MAINT, float* IMBALANCE )
{
	int i, j, k;
	
	printf ("\t\t\tDC Loadflow Results\n");
	printf ("\t\t\t-------------------\n\n\n");
	printf ("\tLine No.\tFrom Bus\tTo Bus\t\tFlows\t\tLoading\n\n");
	
	for (i=1; i<=nl; i++) {
	   float LOAD;
	   k = NNLK[i];
	   j = NNLJ[i];
	   LOAD = ABS(FLOW[i]/LINERATING[i]) * 100;
	   printf("\t%d\t\t%d\t\t%d\t\t%4.1fMW\t\t%3.1f%%",i,k,j,FLOW[i],LOAD);
	   if(MAINT[i]) printf(" M");
	   printf("\n");
	}

	printf ("\n\tBus No.		Imbalance\n\n");
	for (i=1; i<=nn; i++) {
		printf ("\t%d\t\t%4.1fMW\n",i,IMBALANCE[i]);
	}

	printf("\n");
	return;
}


/* This function reads the network parameters in from the study
 * file. It is however very simple, and may NOT detect errors 
 * within the study file */
void ReadParameters (char filename[32], 
int* nn, int* nl, float* PLOAD, float* PGEN, int* NNLK, int* NNLJ, float* XL,
float* LINERATING, BOOL* MAINT )
{
	FILE *record;
	int i;

/* filename is provided by GetStudyName below */	
	record = fopen(filename,"r");

	fscanf(record,"%i %i", nn, nl);

	for (i=1; i<=*nn; i++) {
	   fscanf(record,"%f %f %*f %*f", &PLOAD[i], &PGEN[i]);
	}

	for (i=1; i<=*nl; i++) {
	   char c[80];
	   fscanf(record,"%i %i %f %f %s",
		  &NNLK[i], &NNLJ[i], &XL[i], &LINERATING[i], c);
	   MAINT[i] = (c[0]=='m'||c[0]=='M') ? TRUE : FALSE;
	}
	
	fclose(record);

	return;
}

/* Function obtains a study file name from the program operator */
void GetStudyName (char filename[32])
{
	FILE *record;

	printf ("\nEnter name of study file: ");
	scanf ("%s",filename);
	printf ("\n");

/* Checks to ensuree the file name given is valid */
	record = fopen(filename,"r");

	if (record == NULL) {
	   printf ("\nerror: failed to open study file '%s'\n",filename);
	   exit (0);
	}
}

/* Performs program as described by above functions */		
int main() {
/* The globals below store the network parameters */
int NNLK[NL], NNLJ[NL], nn, nl;
BOOL MAINT[NL];
float LINERATING[NL];

float P[NN];

/* The globals below are used for storing the output data */
float XL[NL], FLOW[NL];
float IMBALANCE[NN];

	{
	char filename[32];
	float PLOAD[NN], PGEN[NN];

	GetStudyName (filename);

	ReadParameters (filename, &nn, &nl, PLOAD, PGEN,
			NNLK, NNLJ, XL, LINERATING, MAINT);

	CalcP (nn, PGEN, PLOAD, P);
        }

	Loadflow (nn, nl, P, NNLK, NNLJ, XL, MAINT, FLOW, IMBALANCE);
	Display (nn, nl, NNLK, NNLJ, FLOW, LINERATING, MAINT, IMBALANCE);

	return 1;
}
