/* *********************************************************************** */
/* NETCHK3   A fast AINDEPENDENT CHECK OF NETWORK CONNECTIVITY             */
/*            - IDENTIFIES nodes WHICH ARE NOT CONNECTED                   */
/* *********************************************************************** */

/* $Revision: 1.5 $*/

/*Modifications (in reverse order)

WBL 22 May 96  BUGFIX ensure maxchain is initailised
WBL 30 Jan 96  Written
*/

#include "loadflow.h"

int nodchain[NN]; int nodchainhd[NN]; int nodchainlast[NN]; int chainsize[NN]; 
int maxsize, maxchain;

inline int root(int node)
{
int next;
for(next = nodchainhd[node]; next!=0 ; ) {
	node = next;
	next = nodchainhd[node];
}
return node;
}

inline int root2(int node, int other)
{
int next;
for(next = nodchainhd[node]; next!=0 ; ) {
	node = next;
	next = nodchainhd[node];
        if(next==other) return -1;
}
return node;
}

inline void connect (int nod1, int nod2)
{
int n1, n2, last_n1;
/* Are they already connected? 
 * Quick check are they adjacent in existing chain ? */

if(nodchain[nod2] == nod1 || nodchain[nod1] == nod2) 
return; 

/* Chase up back pointers, if they are connected their root node 
 * or they encouter each other in the chain (nb can skip each other)
 * will be the same */

if((n1 = root2(nod1,nod2)) < 0 || (n2 = root2(nod2,nod1)) < 0 || (n2 == n1))
return;

/* Not already connected to each other. */

	/* connect two chains together by 
         *inserting n1 as 2nd element of n2 chain */

        last_n1 = (nodchainlast[n1] !=0)? nodchainlast[n1] : n1;

	nodchainhd[n1] = n2;
        nodchain  [last_n1] = nodchain[n2];
        nodchainhd[last_n1] = n2;
	nodchain  [n2] = n1;
	if(nodchainlast[n2] == 0) nodchainlast[n2] = last_n1;

        chainsize[n2] += chainsize[n1];
        if(chainsize[n2] > maxsize) {
		maxsize = chainsize[n2];
		maxchain = n2;
	}

}/*end connect*/

inline BOOL netchk3(int nl, int nn, int NNLK[NL], int NNLJ[NL], BOOL MAINT[NL], 
	     /*outputs */
	     BOOL disconnected[NL])
{
    /* Local variables */
    int next, ib;

    /* Function Body */
    memset(nodchain,0,sizeof(nodchain));
    memset(nodchainhd,0,sizeof(nodchainhd));
    memset(nodchainlast,0,sizeof(nodchainlast));

    for (ib = 1; ib <= nn; ++ib) chainsize[ib] = 1;
    maxsize = 1;
    maxchain = 1;

    for (ib = 1; ib <= nl; ++ib) {
	if (!MAINT[ib]) connect(NNLK[ib],NNLJ[ib]); }

    if(maxsize==nn)
	    return TRUE;

    for (ib = 1; ib <= nn; ++ib) disconnected[ib] = TRUE; /*default*/
    for(next = maxchain; next != 0 ; ) {
	    disconnected[next] = FALSE;
	    next = nodchain[next];
    }

return FALSE;

} /*end netchk */

