/****************************************************************
 *								*
 * Program to 
 *								*
 * File: readfile.c	      				*
 *								*
 * Author: W. Langdon 
 *								*
 * Started: 26 Oct 1995 					*
 *								*
 ****************************************************************/

#define version "13 June 1996 $Revision: 1.14 $"

/*
WBL  24/6/96 Restore calling readbase
WBL  13/6/96 Restore SWales node name file
WBL  20/5/96 Move readnet out of readcont and use 4node file name
WBL   4/5/96 For four node problem
WBL  22/1/96 Make linename and nodename globals
WBL  21/1/96 Add reading swbase.dat
WBL 24/11/95 Check for file open errors in ReadParameters
WBL 21/11/95 Add readcont
WBL  1/11/95 Add readnet
WBL  1/11/95 Cut display_maint also, use printf rather than cout<<
WBL 26/10/95 Cut from gridmain.cc r1.8
*/

#include <assert.h>
/*#include <ctype.h>*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/*
#include <float.h>
#include <time.h>
#include "gridalg.h"
#include "grideval.h"
*/
#include "loadflow.h"

void comment(FILE *record)
{int c;
	for(;(c=getc(record))=='\n';) {
/*		printf("'n' ");*/}
        ungetc(c,record);
	for(;(c=getc(record))=='*';) {
/*		printf("'*' ");*/
		for(;(c=getc(record)),((c!='\n')&&(c!=EOF));){
/*			printf("%d ",c);*/}
	}
/*	printf("*%d '%c'\n",c,c);*/
        ungetc(c,record);
}

void check_sizes(FILE *record, int w, int nn, int nl, int nw)
{
/*printf("check_sizes %d\n", w);*/

       {int nn_, nl_, nw_; 
	comment(record);
	fscanf(record,"%i %i %i", &nn_, &nl_, &nw_);
	if(nn != nn_ || nl != nl_ || nw != nw_) {
		printf("check_sizes: %d missmatch %d %d %d %d %d %d\n",
		       w, nn, nn_, nl, nl_, nw, nw_);
		close(record);
		exit(0);
	}
       }
}/*end check_sizes*/

void read_Q(FILE *record, int nn, double Q[NN*NN/2]    )
{
int j;       
/*printf("read_Q %d\n", nn);*/

for(j=0;j<nn*nn/2;j++) {
	int jj;
	comment(record);
	fscanf(record,"%i %lf", &jj, &Q[j] );
	if((j+1) != jj) {
		printf("read_Q: %d missmatchJ %d %g\n",j,jj,Q[j]);
		exit(0);
	}
/*	printf("%4i  %22.14g\n",j+1, Q[j]); */
}
}/*end read_Q*/

void read_flow(FILE *record, int nl, double flow[NL])
{
int j;       
/*printf("read_flow %d %d\n", nl);*/

for(j=0;j<nl;j++) {
	int jj; char dummy[80]; int invert; 
	comment(record);
	fscanf(record,"%i %s %lf %d", &jj, dummy, &flow[j], &invert );

	if((j+1) != jj) {
		printf("read_flow: %d missmatchJ %d %s %g %d\n",
		       j,jj,dummy,flow[j],invert);
		exit(0);
	}
/*	printf("%4i  %22.14g  %4i\n", j+1, flow[j] ); */
}
}/*end read_flow*/
	
void read_ntovar(FILE *record, int w, int nn, int ntovar[NN])
{
int j;       
/*printf("read_ntovar %d %d\n", w, nn);*/

for(j=0;j<nn;j++) {
	int jj, v; 
	comment(record);
	fscanf(record,"%i %i", &jj, &v);

	if((j+1) != jj) {
		printf("read_ntovar: %d missmatchJ %d %d\n",j,jj,v);
		exit(0);
	}

	if(w==1) ntovar[j]=v;
	else {
		if (ntovar[j] != v) {
			printf("read_ntovar: %d missmatchNV %d\n",j,jj);
			exit(0);
		}
	}
/*	printf("%4i  %4i\n", j+1, ntovar[j]); */
}

}/*end read_ntovar*/

void read_XX(FILE *record, int nn, double xx[NN*2])
{
int j;       
/*printf("read_XX %d %d\n", w, nn);*/

for(j=0;j<nn*2;j++) {
	int jj; 
	comment(record);
	fscanf(record,"%i %lf", &jj, &xx[j] );

	if((j+1) != jj) {
		printf("read_XX: %d missmatchJ %d %g\n",j,jj,xx[j]);
		exit(0);
	}
/*	printf("%4i  %22.14g\n", j+1, xx[j]); */
}

}/*end read_xx*/

void check_susc(FILE *record, int nl, int NNLK[NL], int NNLJ[NL], float XL[NL])
{
int j;       
/*printf("check_susc %d\n", nl);*/

for(j=0;j<nl;j++) {
	int jj; char dummy[80]; int nnlk, nnlj; double f;
	comment(record);
	fscanf(record,"%i %s %i %i %lf", &jj, dummy, &nnlk, &nnlj, &f );

	if((j+1) != jj) {
		printf("check_susc: %d missmatchJ %d %s %d %d %g\n"
		       ,j,jj,dummy,nnlk,nnlj,f);
		exit(0);
	}

/* since managed to pass all these tests with the bug dont bother to keep them!
	if (swco[j]>0) {
	if (NNLK[j+1] != nnlk) {
		printf("check_susc: %d missmatch %d %s %d<>%d %d %g\n"
		       ,j,jj,dummy,NNLK[j+1],nnlk,nnlj,f);
			exit(0);
		}
	if (NNLJ[j+1] != nnlj) {
		printf("check_susc: %d missmatch %d %s %d %d<>%d %g\n"
		       ,j,jj,dummy,nnlk,NNLJ[j+1],nnlj,f);
			exit(0);
		}
        }
	else {
	if (NNLK[j+1] != nnlj) {
		printf("check_susc: %d missmatch %d %s %d %d<>%d %g\n"
		       ,j,jj,dummy,nnlk,NNLK[j+1],nnlj,f);
			exit(0);
		}
	if (NNLJ[j+1] != nnlk) {
		printf("check_susc: %d missmatch %d %s %d<>%d %d %g\n"
		       ,j,jj,dummy,NNLJ[j+1],nnlk,nnlj,f);
			exit(0);
		}
        }
*/
	if (XL[j+1] <(-0.99/f) || (XL[j+1] > -1.01/f )){
		printf("check_susc: %d missmatch %d %s %d %d %f<>%g\n"
		       ,j,jj,dummy,nnlk,nnlj,XL[j+1],f);
			exit(0);
		}

/*	printf("%4i  %4i  %4i  %22.14g\n", j+1, NNLK[j+1],NNLJ[j+1],XL[j+1]);*/
}
}/*end check_susc*/
	

void readbase (char* filename, int nn, int nl, int nw, 
	       /* inputs to cross check file */
	       int NNLK[NL], int NNLJ[NL], float XL[NL], 
	       /* outputs*/
	       double xx[NW][NN*2], double flow[NW][NL], 
	       int ntovar[NN], double Q[NN*NN/2]    )

{
FILE *record;
int w,j;

if((record = fopen(filename,"r"))==NULL) {
	printf("readbase: could not open file %s\n", filename);
	exit(0);
}

check_sizes(record,1,nn,nl,nw);
read_Q(record, nn, Q);
for (w=1; w<=nw; w++) {
	if(w!=1)check_sizes(record,w,nn,nl,nw);
	read_flow(record,nl,flow[w]);
	read_ntovar(record,w,nn,ntovar);
	read_XX(record,nn,xx[w]);
}

/*	for(j=1;j<=nl;j++) printf("%4i  %22.14g\n",j, XL[j]);*/

check_susc(record,nl,NNLK,NNLJ,XL);

fclose(record);

}/*end readbase*/


char* linename[NL]; /* should clean up on exit but ...*/
char* nodename[NN];

void readcont (char* filename, int nn, int nl, int NNLK[NL], int NNLJ[NL],
	       /* outputs*/
	       int* ncontingencies, 
	       BOOL contingency[NCONT][NL], float scale[NCONT] )
{
FILE *record;
int c;
char buffer[80];

if((record = fopen(filename,"r"))==NULL) {
	printf("readcont: could not open file %s\n", filename);
	exit(0);
}
*ncontingencies = 0;
memset(scale,0,sizeof(float)*NCONT);

for (c=1; c<=NCONT; c++) {
	int i, l;
	char* p;
	char* ps;

	do{ if(fgets(buffer,sizeof(buffer),record)==NULL) {
		*ncontingencies = c-1;
		fclose(record);
		return;}
        } while(strlen(buffer)<=3);

	p = &buffer[0];
	ps = p;

	memset(&contingency[c],0,sizeof(BOOL)*NL);

	for (i=1; i<=NL; i++) { /*extract next name from buffer */
		for(;isspace(*p);p++){;};

		if(!isalpha(*p)) break; /* not a name, leave loop */

		ps = p;	
		for(;isalnum(*p);p++){;};
	
		for(l=1;l<=nl;l++) {/* find line index number for name ps */
			if(strncmp(ps,linename[l],(p-ps))==0) goto valid_line;
		}
		printf("readcont: bad name '%s', file %s, contingency %d\n",
		       ps, filename, c);
		exit(0);

valid_line:
		contingency[c][l] = TRUE;
	}
	sscanf(p,"%f",&scale[c]);
	if(scale[c]<=0.0) {
		printf("readcont: bad scaling on %dth cont %s, file %s %f\n",
		       c, ps, filename, scale[c]);
		exit(0);
	}
}
	
printf("readcont: too many contingencies %d %d file %s\n",
       c, NCONT, filename);
exit(0);

}/*end readcont*/

/* This function reads the network parameters in from the study
 * file. It is however very simple, and may NOT detect errors 
 * within the study file */
void ReadParameters (char filename[32], 
int* nn, int* nl, int*nw, int*ncontingencies,
float P[NW][NN], int* NNLK, int* NNLJ, float* XL,
float* LINERATING,
BOOL contingency[NCONT][NL], float scale[NCONT],
double xx[NW][NN*2], double flow[NW][NL], 
int ntovar[NN], double Q[NN*NN/2]
)
{
	FILE *record;
	int w, i;

	if((record = fopen(filename,"r"))==NULL) {
		printf("ReadParameters: could not open file %s\n", filename);
		exit(0);
	}

	fscanf(record,"%i %i %i", nn, nl, nw);

	assert(*nn<NN);
	assert(*nl<NL);
	assert(*nw<NW);
	assert(*nl<*nw);
/*	assert(*nw<=9);*/

	for (w=1; w<=*nw; w++) {
	for (i=1; i<=*nn; i++) {
	   fscanf(record,"%f", &P[w][i]);
	}
	}

	for (i=1; i<=*nl; i++) {
	   char c[80]; 
	   fscanf(record,"%i %i %f %f %*s",
		  &NNLK[i], &NNLJ[i], &XL[i], &LINERATING[i], c);
/*	   MAINT[i] = (c[0]=='m'||c[0]=='M') ? TRUE : FALSE; */

/*NO Dirty frig to avoid recalculating sw.cost */
/*	   LINERATING[i] *= 2.0;            */
	}
	
	fclose(record);

if(!readnet("sw_net.dat", *nn, *nl, NNLK,  NNLJ,
	     /* outputs */
	     linename, nodename) ) exit(0);

/* Setup contingency data here too */
/*                                          */
/*	readcont("sw.cont", *nn, *nl, NNLK, NNLJ, */
/*		 ncontingencies, contingency, scale); */
/*                                           */
/* Setup base data here too */

	readbase("swbase.dat", *nn, *nl, *nw, 
	       /* inputs to cross check file */
	       NNLK, NNLJ, XL, 
	       /* outputs*/
	       xx, flow, ntovar, Q );
}

void display_maint(int nl, int nw, BOOL MAINT[NW][NL])
{
int j, k;

	for(j=1;j<=nw;j++) {
		for(k=1;k<=nl;k++) 
			if(MAINT[j][k]) {printf("%d ",k);}
		if(j<nw) printf(",");
	};
        printf("\n");
}


/* This function reads the network parameters in from the network
 * file. It is however very simple, and may NOT detect errors 
 * within the study file */

BOOL readnet(char filename[32], 
	     int nn, int nl, int NNLK[NL], int NNLJ[NL],
	     /* outputs */
	     char* linename[NL], char* nodename[NN])
{
FILE *record;
int i;
char c[80];

if((record = fopen(filename,"r"))==NULL) {
	printf("readnet: could not open file %s\n", filename);
	return FALSE;
}
fgets(c,sizeof(c),record); /* skip 3 line header */
fgets(c,sizeof(c),record);
fgets(c,sizeof(c),record);

memset(nodename,0,sizeof(char*)*NN);

for (i=1; i<=nl; i++) {
	char* p;
	char* ps;
	do{ if(fgets(c,sizeof(c),record)==NULL) {
		printf("readnet: EOF before line %3d (%s)\n", i, filename);
		return FALSE;
	    }
        } while(strlen(c)<=3);

	p = &c[0];
	ps = p;
	for(;isalnum(*p);p++){;};
	linename[i] = malloc(sizeof(char)*((p-ps)+1));
	strncpy(linename[i],ps,p-ps);

	for(;!isalnum(*p);p++){;};
	ps = p;	
	for(;isalnum(*p);p++){;};
	if(nodename[NNLK[i]]==NULL) {
		nodename[NNLK[i]] = malloc(sizeof(char)*((p-ps)+1));
		strncpy(nodename[NNLK[i]],ps,p-ps);
	}
	else {
		if(strncmp(nodename[NNLK[i]],ps,(p-ps)) != 0) {
			printf("readnet: Line %3d %s; ", i, linename[i]);
			printf("nodeK %3d name missmatch ",NNLK[i]);
			printf("%s <> %s\n", nodename[NNLK[i]], ps);
			return FALSE;
			fclose(record);
		}
	}
 
	for(;!isalnum(*p);p++){;};
	ps = p;
	for(;isalnum(*p);p++){;};
	if(nodename[NNLJ[i]]==NULL) {
		nodename[NNLJ[i]] = malloc(sizeof(char)*((p-ps)+1));
		strncpy(nodename[NNLJ[i]],ps,p-ps);
	}
	else {
		if(strncmp(nodename[NNLJ[i]],ps,(p-ps)) != 0) {
			printf("readnet: Line %3d %s; ", i, linename[i]);
			printf("nodeJ %3d name missmatch ",NNLJ[i]);
			printf("%s <> %s\n", nodename[NNLJ[i]], ps);
			return FALSE;
			fclose(record);
		}
	}
 
}
	
fclose(record);
return TRUE;
}
