/****************************************************************
 *								*
 * Greedy optimal line maintenance scheduler                    *
 *								*
 * File: schedule.c	$Revision: 1.26 $      		       	*
 *								*
 * Author: W. Langdon 
 *								*
 * Started: 19 Nov 1994 					*
 *								*
 ****************************************************************/

#include <stdio.h>
#include <string.h>
#include <float.h>
#include "loadflow.h"

/*#define DEBUG*/

/*Modifications (in reverse order)

WBL 13 Jun 1996 Make compatible with current versions of FitnessWeek 
                for speed up  
WBL 24 Apr 1995 Change for South Wales, ie support nm_weeks
WBL 26 Nov 1994 Move to least increase in Fitness
WBL 26 Nov 1994 Move from Minimum Power flow to Minimum Fitness
*/


char* Schedule_version() {
#ifdef DEBUG
	return "(DEBUG SW least increased cost $Revision: 1.26 $)";
#else
	return "(SW least increased cost $Revision: 1.26 $)";
#endif DEBUG
}


void Display7 (int nl, int w, int line, BOOL MAINT[NL], 
	      float FLOW[NL], float cost, float old_cost)

{
int i;

	printf("%5.0f%5.0f%3d%3d ", old_cost, cost, line, w);
	for (i=1; i<=nl; i++) {
	   printf("%7.0f",FLOW[i]);
	   if(MAINT[i]) printf("M");
	   else         printf(" ");
	}
        printf("\n");

	return;
}
void Schedule (int nn, int nl, int nw, int ncontingencies, float P[NW][NN],
	       int NNLK[NL], int NNLJ[NL], float XL[NL], float LINERATING[NL], 
	       BOOL contingency[NCONT][NL], float scale[NCONT],
	       /* Readonly baseline data */
	       double xx[NW][NN*2], double flow[NW][NL],
	       int ntovar[NN], double Q[NN*NN/2],
	       /* GA Chrome */
	       int order[NL],
	       /* outputs */
	       BOOL MAINT[NW][NL] ) {
int i, w;
float previous_fit[NW], c[NW];
float dummy;
const float S_factor = 5.0; /* should be as grid.fitness.cc */
                  /* sizeof(MAINT) as arg does not work, converts to * !! */
memset(MAINT,0,NW*NL*sizeof(MAINT[0][0])); 

for (w=1; w<=nw; w++) {	
float dummy_flow[NL];
BOOL dummy1; BOOL dummy2[NN];
	previous_fit[w] = 
		FitnessWeek(nn, nl, P[w], NNLK, NNLJ, XL, LINERATING, MAINT[w],
			    Costing, S_factor, FALSE,
			    /*base data*/
			    xx[w], flow[w], ntovar, Q,
			    /* Display outputs */
			    &dummy, &dummy, dummy_flow, &dummy1, dummy2 );
}

for (i=1; i<=nl; i++) {
	int line = order[i];
/* Maintain this line when cost increase is minimum */
/* Find new costs */
	int week = 0;
	float min_increase = FLT_MAX;
	for (w=1; w<=nw; w++) {
		int l;
		BOOL dummy1; BOOL dummy2[NN];
		BOOL test_maint [NL];
		float test_flow [NL];
		for(l=1;l<=nl;l++) {test_maint[l] = MAINT[w][l];}
		test_maint[line] = TRUE;

		c[w] = FitnessWeek(nn, nl, P[w], NNLK, NNLJ, XL, LINERATING, 
				   test_maint,
			    Costing, S_factor, FALSE,
			    /*base data*/
			    xx[w], flow[w], ntovar, Q,
			    /* Display outputs */
			    &dummy, &dummy, test_flow, &dummy1, dummy2 );
#ifdef DEBUG
		Display7( nl, w, line, test_maint, test_flow, 
			  c[w], previous_fit[w] );
#endif
	}
/* Maintain this line in weeks when cost increase is minimum */
	for (w=1; w<=nw-nm_weeks+1; w++) {
		float increase = 0.0;
		int j;
		for(j=w;j<w+nm_weeks;j++){ increase += c[j] - previous_fit[j];}
		if(min_increase > increase) {
			min_increase = increase;
			week = w;
		}
	}
	for (w=week; w<week+nm_weeks; w++) {
		MAINT[w][line] = TRUE;
		previous_fit[w] = c[w];
	}
}

}/*end Schedule */

